@@ -17,6 +17,7 @@
  */
 
 #include <dissimmat.h>
+#include <pthread.h>
 
 extern unsigned char DEB;
 
@@ -58,7 +59,7 @@ void FillMetricMatrixFromSparse(indextype initial_row,indextype final_row,Sparse
    // The mark array is initialized to the marks of rowA
    memcpy(mark,rowmark,ncols);
    
-   // and vb will contains the values of rowB. First, we clean it...
+   // and vb will contain the values of rowB. First, we clean it...
    memset((void *)vb,0x0,ncols*sizeof(counttype));
    
    // and now, we fill its values and update mark adding 2 to the rigth places. In that way, mark will return with 0, 1, 2 or 3
@@ -158,8 +159,9 @@ void FillCosMatrixFromSparse(indextype initial_row,indextype final_row,SparseMat
      }
     }              
    } 
-   
-   D->Set(rowA,rowB,1.0-(prod/(sqrt(sc1)*sqrt(sc2))));
+   // Precision problems in the sqrt calculation might yield an extremely small negative result. we clip it to zero
+   disttype val=1.0-(prod/(sqrt(sc1)*sqrt(sc2)));
+   D->Set(rowA,rowB,val<0.0 ? 0.0 : val);
   }
   
   // This is just to set the main diagonal.
@@ -257,7 +259,7 @@ void FillPearsonMatrixFromSparse(indextype initial_row,indextype final_row,Spars
   
   for (unsigned rowB=0; rowB<rowA; rowB++)
   {
-   // bb will contains the values of rowB. First, we clean it...
+   // bb will contain the values of rowB. First, we clean it...
    memset((void *)vb,0x0,ncols*sizeof(counttype));
    
    // and now, we fill its values
