@@ -40,19 +40,20 @@ extern unsigned char DEB;
 template <typename T>
 void GetJustOneColumnFromFull(std::string fname,indextype nc,indextype nrows,indextype ncols,Rcpp::NumericVector &v)
 {
+
  T *data = new T [nrows]; 
  
  std::ifstream f(fname.c_str());
  // This places us at the nc column of first row (row 0)
- size_t offset=HEADER_SIZE+nc*sizeof(T);
+ std::streampos offset=HEADER_SIZE+nc*sizeof(T);
  for (indextype r=0; r<nrows; r++)
  {
   f.seekg(offset,std::ios::beg);
   // We read one element
-  f.read((char *)(&data[r]),sizeof(T));
+  f.read((char *)(&data[r]),(std::streamsize)sizeof(T));
   // This jumps exactly one row, up to just before the nc column of next row.
   // The number of bytes in one row is the number of columns multiplied by the size of one element.
-  offset += (ncols*sizeof(T));
+  offset += ((std::streampos)ncols*sizeof(T));
  }
  f.close();
  
@@ -68,20 +69,22 @@ void GetManyColumnsFromFull(std::string fname,std::vector<indextype> ncs,indexty
  T data;
  
  std::ifstream f(fname.c_str());
- size_t offset;
+ std::streampos offset;
+ unsigned long long ncsl;
  for (size_t t=0; t<ncs.size(); t++)
  {
+  ncsl=(unsigned long long)ncs[t];
   // This places us at the requested column of the first row
-  offset=HEADER_SIZE+ncs[t]*sizeof(T);
+  offset=HEADER_SIZE+ncsl*sizeof(T);
   for (indextype r=0; r<nrows; r++)
   {
    f.seekg(offset,std::ios::beg);
    // We read one element
-   f.read((char *)(&data),sizeof(T));
+   f.read((char *)(&data),(std::streamsize)sizeof(T));
    m(r,t)=data;
    // This jumps exactly one row, up to just before the nc column of next row.
    // The number of bytes in one row is the number of columns multiplied by the size of one element.
-   offset += (ncols*sizeof(T));
+   offset += ((std::streampos)ncols*sizeof(T));
   }
   
  }
@@ -98,15 +101,16 @@ void GetJustOneColumnFromSparse(std::string fname,indextype nc,indextype nrows,i
  
  // Start of row nr is at the end of former rows, each of them having a different number of elements
  // we must go to the start of each row to find out how many...
- size_t offset=HEADER_SIZE;
+ std::streampos offset=HEADER_SIZE;
  indextype c,ncr;
+ unsigned long long to_add;
  for (indextype r=0; r<nrows; r++)
  {
   f.seekg(offset,std::ios::beg);
   // At the beginning of row r: read how many element there are in it (ncr):
-  f.read((char *)&ncr,sizeof(indextype));
+  f.read((char *)&ncr,(std::streamsize)sizeof(indextype));
   // Read the indices of this row
-  f.read((char *)idata,ncr*sizeof(indextype));
+  f.read((char *)idata,(std::streamsize)ncr*sizeof(indextype));
   // See if the index of the column we are looking for is there (a while loop with premature exit is OK, indices are ordered)
   c=0;
   while ( (c<ncr) && (idata[c]<nc) )
@@ -117,11 +121,12 @@ void GetJustOneColumnFromSparse(std::string fname,indextype nc,indextype nrows,i
   {
    // offset is now the beginning of the current row. We jump the indices (and the number of non-null indices, which is the reason of '+1' in ncr+1)
    // and also jump the first c data, too.
-   f.seekg(offset+((ncr+1)*sizeof(indextype))+(c*sizeof(T)),std::ios::beg);
-   f.read((char *)(&data[r]),sizeof(T));
+   to_add = (ncr+1)*sizeof(indextype)+c*sizeof(T);
+   f.seekg(offset+(std::streampos)to_add,std::ios::beg);
+   f.read((char *)(&data[r]),(std::streamsize)sizeof(T));
   }
   // We advance up to the beginning of next row. The size of this row consists on the nc indices, plus the number of non-null indices, plut the number of present values.
-  offset += ((ncr+1)*sizeof(indextype)+(ncr*sizeof(T)));
+  offset += (((std::streampos)(ncr+1))*sizeof(indextype)+((std::streampos)ncr*sizeof(T)));
  }
  
  f.close();
@@ -139,18 +144,18 @@ template <typename T>
 void GetManyColumnsFromSparse(std::string fname,std::vector<indextype> nc,indextype nrows,indextype ncols,Rcpp::NumericMatrix &m)
 {
  // Differently to the function to get rows, here we need the offsets of absolutely all rows, since at least one element will be extracted from each of them
- std::vector<size_t> offsets(nrows,HEADER_SIZE);
+ std::vector<std::streampos> offsets(nrows,HEADER_SIZE);
  
  std::ifstream f(fname.c_str()); 
- size_t offset=HEADER_SIZE;
+ std::streampos offset=HEADER_SIZE;
  
  indextype ncr;
  for (size_t t=0;t<nrows;t++)
  {
   offsets[t]=offset;
   f.seekg(offset,std::ios::beg);
-  f.read((char *)&ncr,sizeof(indextype));
-  offset += ((ncr+1)*sizeof(indextype)+ncr*sizeof(T));
+  f.read((char *)&ncr,(std::streamsize)sizeof(indextype));
+  offset += (((std::streampos)(ncr+1))*sizeof(indextype)+(std::streampos)ncr*sizeof(T));
  }
  
  // These are temporary arrays to store the indices and values of a row.
@@ -164,11 +169,11 @@ void GetManyColumnsFromSparse(std::string fname,std::vector<indextype> nc,indext
  {
   f.seekg(offsets[t],std::ios::beg);
   // At the beginning of row r: read how many element there are in it (ncr):
-  f.read((char *)&ncr,sizeof(indextype));
+  f.read((char *)&ncr,(std::streamsize)sizeof(indextype));
   // Let's read its indices... No problem with size, ncr is always smaller than ncols
-  f.read((char *)idata,ncr*sizeof(indextype));
+  f.read((char *)idata,(std::streamsize)ncr*sizeof(indextype));
   // ... and let's read the data
-  f.read((char *)data,ncr*sizeof(T));
+  f.read((char *)data,(std::streamsize)ncr*sizeof(T));
    
   // Clear the corresponding column of the matrix to be returned
   for (size_t c=0; c<nc.size(); c++)
@@ -195,27 +200,30 @@ void GetManyColumnsFromSparse(std::string fname,std::vector<indextype> nc,indext
 template <typename T>
 void GetJustOneColumnFromSymmetric(std::string fname,indextype nr,indextype ncols,Rcpp::NumericVector &v)
 {
+ unsigned long long nrl=(std::streampos)nr;
  T *data = new T [ncols]; 
  
  std::ifstream f(fname.c_str());
  
  // This is the beginning of row nr in the binary symmetric data
- size_t offset=HEADER_SIZE+((nr*(nr+1))/2)*sizeof(T);
+ std::streampos offset = HEADER_SIZE + sizeof(T)*(nrl*(nrl+1))/2;
+  
  f.seekg(offset,std::ios::beg);
  
  // Here we read the nr+1 values present in that row, including the (nr,nr) at the main diagonal (which will be normally 0 in a dissimilarity matrix)
- f.read((char *)data,(nr+1)*sizeof(T));
+ f.read((char *)data,(std::streamsize)(sizeof(T)*(nrl+1)));
  
  // The rest of this row is not physically after; we must read the rest of the column that starts in the diagonal, down to the end.
  // It would be clearer to write r=nr+1; r<nrows; r++ but we haven't a variable nrows. But we don't need: symmetric matrices are square.
- offset=HEADER_SIZE+(nr+((nr+1)*(nr+2))/2)*sizeof(T);
+ //offset=HEADER_SIZE+(nr+((nr+1)*(nr+2))/2)*sizeof(T);
+ offset=HEADER_SIZE+sizeof(T)*(nrl+((nrl+1)*(nrl+2))/2);
  for (indextype r=nr+1; r<ncols; r++)
  {
   f.seekg(offset,std::ios::beg);
   // Here we read just one value...
-  f.read((char *)&(data[r]),sizeof(T));
+  f.read((char *)&(data[r]),(std::streamsize)sizeof(T));
   // and advance to the next row, taking into account that each row has a different number of really stored columns, which is r+1
-  offset += ((r+1)*sizeof(T));
+  offset += ((std::streampos)(r+1)*sizeof(T));
  }
  f.close();
  
@@ -235,36 +243,38 @@ void GetManyColumnsFromSymmetric(std::string fname,std::vector<indextype> nr,ind
  T *data = new T [ncols]; 
  
  std::ifstream f(fname.c_str());
- size_t offset;
+ std::streampos offset;
+ unsigned long long nrl;
  for (size_t t=0; t<nr.size(); t++)
  {
   // This is the beginning of row nr in the binary symmetric data
-  offset=HEADER_SIZE+((nr[t]*(nr[t]+1))/2)*sizeof(T);
+  nrl=(unsigned long long)nr[t];
+  offset=HEADER_SIZE+sizeof(T)*((nrl*(nrl+1))/2);
   f.seekg(offset,std::ios::beg);
   // Here we read the nr+1 values present in that row, including the (nr,nr) at the main diagonal (which will be normally 0 in a dissimilarity matrix)
-  f.read((char *)data,(nr[t]+1)*sizeof(T));
+  f.read((char *)data,(std::streamsize)(sizeof(T)*(nrl+1)));
  
   for (size_t c=0; c<nr[t]+1; c++)
    m(c,t)=data[c];                   // This is the difference w.r.t. the original, m is filled transposed
    
   // The rest of this row is not physically after; we must read the rest of the column that starts in the diagonal, down to the end.
   // It would be clearer to write r=nr+1; r<nrows; r++ but we haven't a variable nrows. But we don't need: symmetric matrices are square.
-  offset=HEADER_SIZE+(nr[t]+((nr[t]+1)*(nr[t]+2))/2)*sizeof(T);
+  offset=HEADER_SIZE+sizeof(T)*(nrl+((nrl+1)*(nrl+2))/2);
   for (indextype r=nr[t]+1; r<ncols; r++)
   {
    f.seekg(offset,std::ios::beg);
    // Here we read just one value...
-   f.read((char *)&(data[r]),sizeof(T));
+   f.read((char *)&(data[r]),(std::streamsize)sizeof(T));
    // and advance to the next row, taking into account that each row has a different number of really stored columns, which is r+1
-   offset += ((r+1)*sizeof(T));
+   offset += ((std::streampos)(r+1)*sizeof(T));
   }
   
   for (size_t c=nr[t]+1; c<ncols; c++)
    m(c,t)=data[c];                  // This is the difference w.r.t. the original, m is filled transposed
+  
  }
  
  f.close();
- 
  delete[] data;
 }
 
@@ -543,7 +553,7 @@ Rcpp::NumericVector GetJColByName(std::string fname,std::string colname)
 //' Returns (as a R numeric matrix) the columns with the requested column names from the matrix contained in a jmatrix binary file
 //'
 //' @param fname        String with the file name that contains the binary data.
-//' @param extcolnames  A numeric vector with the names of the columns to be extracted. If the binary file has no column names, or _any_ of the column names is not present, an empty matrix is returned.
+//' @param extcolnames  A vector of RStrings with the names of the columns to be extracted. If the binary file has no column names, or _any_ of the column names is not present, an empty matrix is returned.
 //' @return             A numeric matrix with the values of elements in the requested columns
 //' @examples
 //' Rf <- matrix(runif(48),nrow=6)
